#
# See the file "LICENSE" for the full license governing this code.
#
# frozen_string_literal: true

newproperty(:local_undo) do
  include EasyType
  include PuppetX::EnterpriseModules::Oracle::Access
  include PuppetX::EnterpriseModules::Oracle::Information

  to_translate_to_resource do |raw_resource|
    return nil if raw_resource['STATE'] != 'running' || pdb?(raw_resource['SID'])

    local_undo = database_properties(raw_resource['SID']).find { |e| e['PROPERTY_NAME'] == 'LOCAL_UNDO_ENABLED' }&.fetch('VALUE') if raw_resource['STATE'] == 'running'
    case local_undo
    when 'TRUE'
      'on'
    when 'FALSE', nil
      'off'
    end
  end

  desc <<-DESC
    Local undo status.

    In Oracle Database 12c Release 1 all containers in an instance shared the same undo tablespace. In Oracle 12c Release 2 and higher versions each container in an instance can
    use its own undo tablespace. This new undo management mechanism is called local undo mode, whilst that of previous releases is now known as shared undo mode.
    Local undo mode is the default mode in newly created databases, so you will probably only need to consider switching undo modes for upgraded instances.

    You should switch to local undo mode unless you have a compelling reason not to. Some of the new multitenant features in 12.2 rely on local undo.#{' '}

    Here is an example:

        ora_database{'dbname':
          ...
          container_database => 'enabled',
          local_undo         => 'on',
          ...
        }

    **WARNING**: In order to change this value on a running database, Puppet will stop and restart the database, thus creating unavailability.

  DESC

  newvalues(:on, :off)
  data_type("Optional[Enum['on', 'off']]")

  on_modify do
    statement = template('ora_config/ora_database/local_undo.sql.erb', binding)
    sql(statement, :sid => instance)
  end

  def insync?(is)
    return true if resource.contained_by

    super
  end
end
