# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#

newproperty(:db_unique_name) do
  include EasyType
  include PuppetX::EnterpriseModules::Oracle::Access

  desc <<-DESC
    Manage rman configuration for `DB_UNIQUE_NAME`.
    The configuration can be set to default by setting it to 'clear' (case insensitive).
    NOTE you cannot have a connect identifier called 'CLEAR'.

    ora_rman_config { 'SID':
      ...
      db_unique_name => {
        'DB_UNIQUE_NAME'         => 'CONNECT_IDENTIFIER',
        'DB_UNIQUE_NAME_STANDBY' => 'CONNECT_IDENTIFIER_STANDBY',
        'OLD_DB_UNIQUE_NAME'     => 'CLEAR',
      },
      ...
    }
  DESC

  data_type('Optional[Hash[String[1],Sting[1]]]')

  to_translate_to_resource do |raw_resource|
    raw_resource['DB_UNIQUE_NAME']
  end

  def insync?(is)
    return true if is == should

    checkdiff(is, should).empty?
  end

  def change_to_s(currentvalue, newvalue)
    diff = checkdiff(currentvalue, newvalue)
    messages = []
    diff.each_pair do |dbun, conn_identifier|
      messages << if conn_identifier.casecmp('clear').zero?
                    "clearing db_unique_name #{dbun}"
                  else
                    "changing db_unique_name #{dbun} to #{conn_identifier}"
                  end
    end
    messages.join(', ')
  end

  on_modify do
    diff = checkdiff(provider.db_unique_name, should)
    diff.each_pair do |unique_name, conn_identifier|
      if conn_identifier.to_s.casecmp('clear').zero?
        rman("configure db_unique_name '#{unique_name}' clear", { :sid => resource.sid, :target_conn => resource.target_conn, :catalog_conn => resource.catalog_conn })
      else
        rman("configure db_unique_name '#{unique_name}' connect identifier '#{conn_identifier}'", { :sid => resource.sid, :target_conn => resource.target_conn, :catalog_conn => resource.catalog_conn })
      end
    end
  end

  def checkdiff(is, should)
    my_is = is.to_s == 'absent' ? {} : is
    my_should = should
    my_should.each_pair do |key, val|
      if val.to_s.casecmp('clear').zero?
        my_should.delete(key) unless my_is.key?(key)
      elsif my_is[key].to_s.casecmp(val.to_s).zero?
        my_should.delete(key)
      end
    end
  end
end
