# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'tmpdir'
require 'puppet_x/enterprisemodules/oracle/settings'
module PuppetX
  module EnterpriseModules
    module Oracle
      # docs
      class Command
        include Settings

        DEFAULT_TIMEOUT ||= 300 # 5 minutes

        COMMAND_VALID_OPTIONS ||= [
          :sid,
          :os_user,
          :password,
          :timeout,
          :username
        ].freeze

        attr_reader :sid, :os_user, :password, :timeout, :username

        def initialize(command, options, valid_options = COMMAND_VALID_OPTIONS)
          @valid_options = valid_options
          check_options(options)
          @command  = command
          @password = options[:password] # may be empty
          @timeout  = options.fetch(:timeout) { DEFAULT_TIMEOUT }
          @sid      = options.fetch(:sid) { raise ArgumentError, 'you need to specify a sid for oracle access' }
          @os_user  = options.fetch(:os_user) { configuration_value_for(@sid, 'os_user') }
          @username = options.fetch(:username, nil)
        end

        def command_string(arguments = '')
          config = @configuration.fetch(@sid) { fail "#{sid} not found in settings" }
          oracle_home = config.fetch('oracle_home') { fail "oracle_home not specified for #{sid}" }
          # TODO: what mechanisme do we need for Windows
          if Puppet::Util::Platform.windows?
            # TODO: Do we need to set ORACLE_HOME?
            "cd #{working_dir} & set ORACLE_SID=#{@host_sid} & #{@command} #{arguments}\""
          else
            "su - #{@os_user} -c \"cd #{working_dir}; export ORACLE_SID=#{@sid}; export ORACLE_HOME=#{oracle_home}; export PATH=$PATH:#{oracle_home}/bin; #{@command} #{arguments}\""
          end
        end

        def execute(arguments)
          options = { :failonfail => true, :combine => true }
          value = ''
          within_time(@timeout) do
            Puppet.debug "command.rb: Executing #{@command} command: #{arguments} on #{@sid} as #{os_user}, connected as #{username}"
            value = if Puppet::Util::Platform.windows?
                      `command_string(arguments)`
                    else
                      Puppet::Util::Execution.execute(command_string(arguments), options)
                    end
          end
          value
        end

        protected

        def working_dir
          pwd = Dir.pwd
          unless writable_by_user(@os_user, pwd)
            pwd = Dir.tmpdir
            Puppet.debug "command.rb: Running Oracle command from a pwd that is not writable for os user #{os_user} therefore resetting pwd to '#{pwd}'"
          end
          pwd
        end

        private

        def writable_by_user(os_user, pwd)
          # TODO: what mechanisme do we need for Windows
          if Puppet::Util::Platform.windows?
            true
          else
            system "su - #{os_user} -c \"cd #{pwd};\" > /dev/null 2>&1"
          end
        end

        def within_time(timeout, &block)
          Puppet.debug "command.rb: Using timeout #{timeout}"
          if timeout.zero?
            yield
          else
            Timeout.timeout(timeout, &block)
          end
        end

        def check_options(options)
          options.each_key { |key| raise ArgumentError, "option #{key} invalid for #{@command}. Only #{@valid_options.join(', ')} are supported" unless @valid_options.include?(key) }
        end
      end
    end
  end
end
