# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
module PuppetX
  module EnterpriseModules
    module Oracle
      # Docs
      module Information
        def self.included(parent)
          parent.extend(Information)
        end

        def cached_sid_value(name, sid)
          # rubocop:disable Style/ClassVars
          @@cache ||= {}
          # rubocop:enable Style/ClassVars
          @@cache[name] ||= {}
          if @@cache[name][sid].nil?
            @@cache[name][sid] = yield
          else
            Puppet.debug "Using cached version of #{name} for sid #{sid}."
            @@cache[name][sid]
          end
        end

        def oracle_managed_files_enabled?(on_sid = sid)
          !db_create_file_dest(on_sid).to_s.empty?
        end

        def cluster?(on_sid = sid)
          cached_sid_value(:cluster, on_sid) { sql('select parallel as par from v$instance', :sid => on_sid).first['PAR'] == 'YES' }
        end

        def containerdb?(on_sid = sid)
          oracle_major_version(on_sid) >= 12 && cached_sid_value(:containerdb, on_sid) { sql('SELECT CDB FROM V$DATABASE', :sid => on_sid).first['CDB'] == 'YES' }
        end

        def rootdb?(on_sid = sid)
          return false unless containerdb?(on_sid)

          oracle_major_version(on_sid) >= 12 && cached_sid_value(:rootdb, on_sid) { sql("select sys_context('userenv', 'con_id') as con_id from dual", :sid => on_sid).first['CON_ID'] == '1' }
        end

        def seeddb?(on_sid = sid)
          return false unless containerdb?(on_sid)

          oracle_major_version(on_sid) >= 12 && cached_sid_value(:seeddb, on_sid) { sql("select sys_context('userenv', 'con_id') as con_id from dual", :sid => on_sid).first['CON_ID'] == '2' }
        end

        def pdb?(on_sid = sid)
          return false unless containerdb?(on_sid)

          oracle_major_version(on_sid) >= 12 && cached_sid_value(:pdb, on_sid) { sql("select sys_context('userenv', 'con_id') as con_id from dual", :sid => on_sid).first['CON_ID'].to_i > 2 }
        end

        #
        # Don't cache this one. This might change during a run and we always need the latest
        # value
        #
        def open_pdbs(on_sid = sid)
          return [] unless containerdb?(on_sid)

          sql("select name from v$pdbs where open_mode != 'MOUNTED'", :sid => on_sid).map { |e| e['NAME'] }
        end

        def primary?(on_sid = sid)
          cached_sid_value(:primary, on_sid) { sql('select database_role from v$database', :sid => on_sid).first['DATABASE_ROLE'] == 'PRIMARY' }
        end

        def cluster_instances(on_sid = sid)
          cached_sid_value(:cluster_instances, on_sid) { sql('select INSTANCE_NAME from gv$instance', :sid => on_sid).collect { |e| e['INSTANCE_NAME'] } }
        end

        def database_version(on_sid = sid)
          cached_sid_value(:database_version, on_sid) { sql('select version from v$instance', :sid => on_sid).first['VERSION'] }
        end

        def db_domain(on_sid = sid)
          cached_sid_value(:db_domain, on_sid) { sql("select value from v$parameter where name = 'db_domain'", :sid => on_sid).first['VALUE'] }
        end

        def db_create_file_dest(on_sid = sid)
          cached_sid_value(:db_create_file_dest, on_sid) { sql("select value from v$parameter where name = 'db_create_file_dest'", :sid => on_sid).first['VALUE'] }
        end

        def diagnostic_dest(on_sid = sid)
          cached_sid_value(:diagnostic_dest, on_sid) { sql("select value from v$parameter where name = 'diagnostic_dest'", :sid => on_sid).first['VALUE'] }
        end

        def value_for_init_param(init_param, on_sid = sid)
          cached_sid_value(init_param, on_sid) { sql("select value from v$parameter where name = '#{init_param}'", :sid => on_sid).first['VALUE'] || '' }
        end

        def database_properties(on_sid = sid)
          cached_sid_value(:database_properties, on_sid) { sql('select property_name, property_value as value from database_properties', :sid => on_sid) }
        end

        def oracle_major_version(on_sid = sid)
          database_version(on_sid).split('.').first.to_i
        end

        def sid_for(on_sid = sid)
          pdb?(on_sid) ? configuration_for(on_sid)['contained_by'] : on_sid
        end

        def db_for(on_sid = sid)
          cluster?(on_sid) ? sid_for(on_sid).chop : sid_for(on_sid)
        end

        def local_sid_for_db(db)
          running_sids.find { |sid| sid.include? db }
        end
      end
    end
  end
end
