# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
# Docs
require 'puppet_x/enterprisemodules/oracle/information'
module PuppetX
  module EnterpriseModules
    module Oracle
      # rubocop:disable Style/Documentation
      module Settings
        # rubocop:enable Style/Documentation
        include PuppetX::EnterpriseModules::Oracle::Information

        ASM_REGXP       ||= /^(\+ASM|default_asm)\d*$/.freeze
        MGMT_REGXP      ||= /^-MGMTDB\d*$/.freeze
        NON_ASM_REGXP   ||= /^(?:(?!(\+ASM|-MGMTDB|default_asm)\d*).)*$/.freeze

        def settings_file
          path = if Puppet::Util::Platform.windows?
                   'c:\\Windows\\System32\\Drivers\\etc\ora_setting.yaml'
                 else
                   '/etc/ora_setting.yaml'
                 end
          Pathname.new(path).expand_path
        end

        def self.included(parent)
          parent.extend(Settings)
        end

        def configuration_for(key)
          fail "No configuration found for #{key}." if configuration.nil? || configuration.key?(key).nil? || !configuration.key?(key)

          configuration[key]
        end

        def configuration_value_for(sid, value, default_value = '')
          configuration_for(sid).fetch(value) { default_value }
        end

        def configuration
          @configuration = read_from_yaml
        end

        def read_from_yaml
          if File.exist?(settings_file)
            # rubocop:disable Security/Open
            # rubocop:disable Security/YAMLLoad
            open(settings_file) { |f| YAML.load(f) }
            # rubocop:enable Security/Open
            # rubocop:enable Security/YAMLLoad
          else
            {}
          end
        end

        def asm_sid?(sid)
          sid =~ ASM_REGXP
        end

        def asm_sids
          registered_sids.select { |sid| asm_sid?(sid) }
        end

        def mgmt_sid?(sid)
          sid =~ MGMT_REGXP
        end

        def mgmt_sids
          registered_sids.select { |sid| mgmt_sid?(sid) }
        end

        def database_sid?(sid)
          sid =~ NON_ASM_REGXP
        end

        def database_sids
          registered_sids.select { |sid| database_sid?(sid) }
        end

        def valid_sid?(sid)
          registered_sids.include?(sid)
        end

        def valid_asm_sid?(sid)
          asm_sids.include?(sid)
        end

        def valid_database_sid(sid)
          database_sids.include?(sid)
        end

        def registered_sids
          if configuration.is_a?(Hash)
            configuration.keys
          else
            []
          end
        end

        # Is the SID a local running sid
        def remote_sid?(sid)
          connect_string = configuration_value_for(sid, 'connect_string')
          #
          # Connect string is both used when accessing a remote database and when accessing
          # a container database. When we access a container database, the connect string
          # contains the local node name. So this is the way to distinquesh between a
          # pluggable database and a real remote database.
          #
          connect_string != '' && !connect_string.include?(Facter.value('hostname'))
        end

        def local_pdb?(sid)
          pdb = configuration_value_for(sid, 'pluggable')
          return false unless pdb

          container = configuration_value_for(sid, 'contained_by')
          connect_string = configuration_value_for(sid, 'connect_string')
          connect_string.empty? && running_pdb?(sid) ? true : connect_string.include?(Facter.value('hostname')) && running_db?(container) && running_pdb?(sid)
        end

        def running_pdb?(sid)
          container = configuration_value_for(sid, 'contained_by')
          open_pdbs(container).include?(sid)
        end

        def container_db?(sid)
          configuration_value_for(sid, 'cdb')
        end

        def normal_db?(sid)
          !configuration_value_for(sid, 'pluggable')
        end

        def running_db?(sid)
          if Facter.value(:kernel) == 'windows'
            `tasklist /svc | findstr "oracle.exe.*OracleService#{sid}"` != ''
          else
            `ps -ef | egrep "(ora|xe)_pmon_#{sid}$"` != ''
          end
        end

        def running_sids
          running_database_sids + running_asm_sids + running_mgmt_sids
        end

        def running_database_sids
          database_sids.select do |sid|
            process?(db_process_regexp(sid)) || remote_sid?(sid) || local_pdb?(sid)
          end
        end

        def running_nopdb_database_sids
          database_sids.select do |sid|
            process?(db_process_regexp(sid)) || remote_sid?(sid)
          end
        end

        def running_primary_database_sids
          running_database_sids.select { |sid| primary?(sid) }
        end

        # Array of normal database that are running
        # Might not be needed after all, we'll see
        def running_normal_database_sids
          running_database_sids.select do |sid|
            process?(db_process_regexp(sid)) && normal_db?(sid)
          end
        end

        # Array of multitenant database that are running
        # Might not be needed after all, we'll see
        def running_mt_database_sids
          running_database_sids.select do |sid|
            container_db?(sid) || local_pdb?(sid)
          end
        end

        def running_asm_sids
          asm_sids.select do |sid|
            process?(asm_process_regexp(sid)) || remote_sid?(sid)
          end
        end

        def running_mgmt_sids
          mgmt_sids.select do |sid|
            process?(mgmt_process_regexp(sid)) || remote_sid?(sid)
          end
        end

        def default_sids
          configuration.select { |_sid, hash| hash['default'] == true }.to_h.keys
        end

        def num_default_database_sids
          database_sids.count { |sid| default_sids.include? sid }
        end

        def num_default_asm_sids
          database_sids.count { |sid| default_sids.include? sid }
        end

        def default_database_sid
          database_sids.find { |sid| default_sids.include? sid }
        end

        def default_asm_sid
          asm_sids.find { |sid| default_sids.include? sid }
        end

        private

        def process?(regexp)
          command = if Puppet::Util::Platform.windows?
                      "tasklist /svc | findstr \"#{regexp}\""
                    else
                      "ps -ef | egrep \"#{regexp}\" | grep -v grep"
                    end
          Puppet.debug "settings.rb: Command is #{command}"
          output = `#{command}`
          Puppet.debug "settings.rb: #{output}"
          value = output != ''
          Puppet.debug "Returned value is: #{value}"
          value
        end

        def db_process_regexp(sid)
          if Puppet::Util::Platform.windows?
            "oracle.exe.*OracleService#{sid}"
          else
            "(ora|xe)_pmon_#{sid}$"
          end
        end

        def asm_process_regexp(sid)
          "asm_pmon_\\\\#{sid}$"
        end

        def mgmt_process_regexp(sid)
          "mdb_pmon_\\\\#{sid}$"
        end
      end
    end
  end
end
