# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/oracle/command'
require 'puppet_x/enterprisemodules/oracle/settings'
require 'easy_type'
require 'easy_type/encryption'
# rubocop: disable Style/ClassVars

module PuppetX
  module EnterpriseModules
    module Oracle
      # Docs<<<<<<< HEAD
      class SqlplusCommand < ::PuppetX::EnterpriseModules::Oracle::Command
        include ::EasyType::Template
        include Settings
        include ::EasyType::Encryption

        @@created_files ||= []

        SQLPLUS_VALID_OPTIONS ||= [
          :sid,
          :os_user,
          :password,
          :timeout,
          :username,
          :daemonized,
          :failonsqlfail, # Don't fail if the sql fails
          :catch_errors,
          :parse, # When parse is false, the output is not mment for parsing
          :catch_extra_errors
        ].freeze

        attr_reader :sid, :os_user, :password, :timeout, :username, :daemonized, :failonsqlfail, :catch_errors, :parse, :catch_extra_errors

        def initialize(options = {})
          @failonsqlfail     = options.fetch(:failonsqlfail, true)
          @catch_errors      = options.fetch(:catch_errors) { /SP\d-\d{4}:.*/ }
          @catch_extra_errors = options[:catch_extra_errors]
          @catch_errors      = Regexp.union(@catch_errors, @catch_extra_errors) if @catch_extra_errors
          @parse             = options.fetch(:parse, true)
          super('sqlplus -S /nolog ', options, SQLPLUS_VALID_OPTIONS)
          #
          # Set all values based on settings
          #
          @sys_username           = configuration_value_for(@sid, 'user')
          @encrypted_sys_password = configuration_value_for(@sid, 'password')
          @sys_password           = @sys_password.nil? || @sys_password.empty? ? decrypted_value(@encrypted_sys_password) : ''
          # rubocop:disable Lint/UselessAssignment
          os_user                 = configuration_value_for(@sid, 'os_user')
          # rubocop:enable Lint/UselessAssignment
          encrypted_os_user_pwd   = configuration_value_for(@sid, 'os_user_pwd')
          @os_user_pwd            = @os_user_pwd.nil? || @os_user_pwd.empty? ? decrypted_value(encrypted_os_user_pwd) : ''
          @oracle_home            = configuration_value_for(@sid, 'oracle_home')
          @settings_cdb           = configuration_value_for(@sid, 'cdb')
          @syspriv                = configuration_value_for(@sid, 'syspriv')
          @contained_by           = configuration_value_for(@sid, 'contained_by')
          @daemonized             = options.fetch(:daemonized) { configuration_value_for(@sid, 'daemonized') }
          @daemonized = false if @daemonized.to_s.empty?
          @host_sid = if @contained_by.nil? || @contained_by.empty?
                        @sid
                      else
                        @contained_by
                      end
          #
          # Validate values
          #
          fail "Invalid password specified for user #{@sys_username}" if @sys_password.include? '@'

          connect_string = configuration_value_for(@sid, 'connect_string')
          if connect_string.empty?
            @connect_string = ''
          else
            if @contained_by.nil? || @contained_by.empty?
              node = connect_string.split(/\W/).first
              ::Entitlement::Entitlement.entitled?("ora_remote_access##{node}#access")
            end
            @connect_string = "@#{connect_string}"
          end
        end

        def command_string(arguments = '')
          "su - #{@os_user} -c \"cd #{working_dir};export PERL5LIB=$(ls -d #{@oracle_home}/perl/lib/[0-9]*);export NLS_LANG=english;export ORACLE_SID=#{@host_sid};export ORACLE_HOME=#{@oracle_home};unset TNS_ADMIN;unset TWO_TASK;export PATH=#{@oracle_home}/bin:$PATH; #{@command} #{arguments}\""
        end

        def execute(arguments, powershell_script = '')
          options = { :failonfail => true, :combine => true }
          value = ''
          command = if Puppet::Util::Platform.windows?
                      %(powershell -File "#{powershell_script}")
                    else
                      command_string(arguments)
                    end
          within_time(@timeout) do
            Puppet.debug "sqlplus_command.rb: Executing #{@command} command: #{arguments} as user #{os_user} outside of the daemon"
            value = Puppet::Util::Execution.execute(command, options)
            Puppet.debug "sqlplus_command.rb: Output:\n #{value}"
          end
          value.encode!('UTF-8', :invalid => :replace, :undef => :replace)
          output_file = Tempfile.new(['sql', '.log'])
          ObjectSpace.undefine_finalizer(output_file) # Don't delete the file
          output_file.write(value)
          output_file.close
          FileUtils.chown(@os_user, nil, output_file.path)
          FileUtils.chmod(0o600, output_file.path)
          FileUtils.chown(@os_user, nil, output_file.path)
          FileUtils.chmod(0o600, output_file.path)
          path = output_file.path
          @@created_files << path
          Puppet.debug "sqlplus_command.rb: SQL output saved to #{output_file.path}"
          scan_for_errors(value) if @failonsqlfail
          value
        end

        def execute_sql_command(command, output_file)
          #
          # Do the stuff
          #
          Puppet.debug "sqlplus_command.rb: Executing sql statement :\n #{command}"
          script = command_file(template('ora_config/ora_config/execute.sql.erb', binding))
          if Puppet::Util::Platform.windows?
            # rubocop:disable Lint/UselessAssignment
            powershell_script = command_file(template('ora_config/ora_config/sqlplus_command.ps1.erb', binding), type = '.ps1')
            # rubocop:enable Lint/UselessAssignment
            Puppet.debug "sqlplus_command.rb: Using powershell script: #{powershell_script}"
          end
          if username
            Puppet.debug "sqlplus_command.rb: Connecting to oracle_sid #{@sid} with connect_string \"#{@connect_string}\" as user #{username} and privilege #{@syspriv}"
          else
            Puppet.debug "sqlplus_command.rb: Connecting to oracle_sid #{@sid} with connect_string \"#{@connect_string}\" as user #{@sys_username} and privilege #{@syspriv}"
          end
          execute("@#{script}", powershell_script)
          content = File.read(output_file)
          content.encode!('UTF-8', :invalid => :replace, :undef => :replace)
          content
        end

        private

        def scan_for_errors(string)
          fail "error in sql statement processing\n #{string}" if string.scan(@catch_errors).any?
        end

        def command_file(content, type = '.sql')
          command_file = Tempfile.new(['command', type])
          ObjectSpace.undefine_finalizer(command_file) # Don't delete the file
          command_file.write(content)
          command_file.close
          FileUtils.chown(@os_user, nil, command_file.path)
          FileUtils.chmod(0o600, command_file.path)
          FileUtils.chown(@os_user, nil, command_file.path)
          FileUtils.chmod(0o600, command_file.path)
          path = command_file.path
          @@created_files << path
          path
        end

        at_exit do
          #
          # If environment variable ORA_CONFIG_KEEP is set to yes,
          # we won't remove the files else we will.
          #
          @@created_files.each { |cmd_file| FileUtils.rm(cmd_file) } unless ENV['ORA_CONFIG_KEEP'].to_s.casecmp('yes').zero?
        end
      end
    end
  end
end
# rubocop: enable Style/ClassVars
