# frozen_string_literal: true

require 'easy_type'

module PuppetX
  module EnterpriseModules
    module Oracle
      # rubocop:disable Style/Documentation
      class SqlplusDaemon < SqlplusCommand
        # rubocop:enable Style/Documentation
        def initialize(options = {})
          super
          @catch_errors = Regexp.union(@catch_errors, /ORA-\d+:.*$/) if @failonsqlfail
          @daemon = ::EasyType::Daemon.run(identity)
          start_daemon if @daemon.nil?
        end

        def execute_sql_command(command, output_file, timeout = nil)
          timeout ||= @timeout
          ensure_connection
          begin
            Puppet.debug "Executing daemonized sql-command #{command}"
            @daemon.execute_command template('ora_config/ora_config/daemon_execute.sql.erb', binding)
            @daemon.sync(timeout) do |line|
              raise ThreadError if /ORA-03135|ORA-03114/.match?(line)

              if line&.match?(@catch_errors)
                @daemon.sync(timeout) # Clear buffer till end string
                fail "Error in execution of SQL command: #{command}.\nFound error #{line}"
              end
            end
          rescue ThreadError
            @retries = @retries.nil? ? 1 : @retries + 1
            fail 'maximum number of retries for daemon restart exhausted.' if @retries == 3

            Puppet.debug 'Detected disconnected daemon. restarting it...'
            @daemon.kill
            start_daemon
            retry
          end
          File.read(output_file)
        end

        private

        def start_daemon
          #
          # We don't need the sudo part of the returned command string, so
          # we remove it
          #
          @command = command_string.split('"')[1]
          Puppet.info "Starting the Sqlplus daemon with os user #{@os_user} on sid #{@sid}"
          @daemon = ::EasyType::Daemon.new(identity, @command, @os_user)
          connect_to_oracle
        end

        #
        # This functions ensure's that the sqlplus proces is still running. Sometimes,
        # for example when Oracle is beeing patched, the sqlplus process in the daemon stops.
        # This also means that sqlplus will not detect a ORA-03135 error because there is no sqlplus.
        # To fix this, for EVERY sql command we push to the daemon, we first send an empty
        # command to the daemon, with a short timeout (2 seconds). It just does a PROMPT.
        # If the sqlplus process is NOT running, it will stop the daemon and restart a new one.
        #
        def ensure_connection
          Puppet.debug 'Validating sql daemon'
          @daemon.execute_command template('ora_config/ora_config/validate_daemon.sql.erb', binding)
          loop do
            line = @daemon.send(:timed_readline, 2)
            break if line.include?('~~~~COMMAND SUCCESSFUL~~~~')
          end
        rescue StandardError
          Puppet.debug 'Detected daemon with sqlplus process. Restarting...'
          @daemon.kill
          start_daemon
        end

        def identity
          "ora-#{@sid}"
        end

        def connect_to_oracle
          Puppet.debug "Connecting to Oracle sid in daemon #{@sid} with os_user #{@os_user}"
          @daemon.execute_command template('ora_config/ora_config/daemon_connect.sql.erb', binding)
        end
      end
    end
  end
end
